@@ -325,12 +325,20 @@ gedit_notebook_reorder_tab (GeditNotebook *src,
 			    GeditTab      *tab,
 			    gint           dest_position)
 {
+	gint old_position;
+	
 	g_return_if_fail (GEDIT_IS_NOTEBOOK (src));	
 	g_return_if_fail (GEDIT_IS_TAB (tab));
 
+	old_position = gtk_notebook_page_num (GTK_NOTEBOOK (src), 
+				    	      GTK_WIDGET (tab));
+				    	      
+	if (old_position == dest_position)
+		return;
+
 	gtk_notebook_reorder_child (GTK_NOTEBOOK (src), 
-					    GTK_WIDGET (tab), 
-					    dest_position);
+				    GTK_WIDGET (tab),
+				    dest_position);
 		
 	if (!src->priv->drag_in_progress)
 	{
@@ -982,22 +990,21 @@ remove_tab (GeditTab      *tab,
 {
 	GtkWidget *label, *ebox;
 	gint position;
-	
+
 	position = gtk_notebook_page_num (GTK_NOTEBOOK (nb), GTK_WIDGET (tab));
-	
+
 	label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), GTK_WIDGET (tab));
 	ebox = GTK_WIDGET (g_object_get_data (G_OBJECT (label), "label-ebox"));
 	gedit_tooltips_set_tip (GEDIT_TOOLTIPS (nb->priv->title_tips), 
 			        ebox, 
 			        NULL, 
 			        NULL);
-					      
+
 	g_signal_handlers_disconnect_by_func (tab,
 					      G_CALLBACK (sync_name), 
 					      label);
 
-	/**
-	 * we ref the tab so that it's still alive while the tabs_removed
+	/* we ref the tab so that it's still alive while the tabs_removed
 	 * signal is processed.
 	 */
 	g_object_ref (tab);
