@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-plugins-engine.c,v 1.30 2006/05/02 10:07:47 sfre Exp $
+ * $Id: gedit-plugins-engine.c 5517 2007-02-25 18:44:03Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -60,26 +60,31 @@ typedef enum
 {
 	GEDIT_PLUGIN_LOADER_C,
 	GEDIT_PLUGIN_LOADER_PY,
-} GeditPluginLang;
+} GeditPluginLoader;
 
 struct _GeditPluginInfo
 {
-	gchar        *file;
+	gchar             *file;
 	
-	gchar        *location;
-	GeditPluginLang lang;
-	GTypeModule  *module;
+	gchar             *location;
+	GeditPluginLoader  loader;
+	GTypeModule       *module;
 
-	gchar        *name;
-	gchar        *desc;
-	gchar        *icon_name;
-	gchar       **authors;
-	gchar        *copyright;
-	gchar        *website;
+	gchar             *name;
+	gchar             *desc;
+	gchar             *icon_name;
+	gchar            **authors;
+	gchar             *copyright;
+	gchar             *website;
 	 
-	GeditPlugin  *plugin;
+	GeditPlugin       *plugin;
 	
-	gboolean     active;
+	gint               active : 1;
+	
+	/* A plugin is unavailable if it is not possible to activate it
+	   due to an error loading the plugin module (e.g. for Python plugins
+	   when the interpreter has not been correctly initializated) */
+	gint               available : 1;
 };
 
 static void	gedit_plugins_engine_active_plugins_changed (GConfClient *client,
@@ -91,7 +96,32 @@ static GList *gedit_plugins_list = NULL;
 
 static GConfClient *gedit_plugins_engine_gconf_client = NULL;
 
-GSList *active_plugins = NULL;
+static GSList *active_plugins = NULL;
+
+static void
+gedit_plugin_info_free (GeditPluginInfo *info)
+{
+	if (info->plugin != NULL)
+	{
+	       	gedit_debug_message (DEBUG_PLUGINS, "Unref plugin %s", info->name);
+
+		g_object_unref (info->plugin);
+		
+		/* info->module must not be unref since it is not possible to finalize 
+		 * a type module */
+	}
+
+	g_free (info->file);
+	g_free (info->location);
+	g_free (info->name);
+	g_free (info->desc);
+	g_free (info->icon_name);
+	g_free (info->website);
+	g_free (info->copyright);
+	g_strfreev (info->authors);
+
+	g_free (info);
+}
 
 static GeditPluginInfo *
 gedit_plugins_engine_load (const gchar *file)
@@ -140,7 +170,8 @@ gedit_plugins_engine_load (const gchar *file)
 				     "Gedit Plugin",
 				     "Module",
 				     NULL);
-	if (str)
+
+	if ((str != NULL) && (*str != '\0'))
 	{
 		info->location = str;
 	}
@@ -157,16 +188,16 @@ gedit_plugins_engine_load (const gchar *file)
 				     NULL);
 	if (str && strcmp(str, "python") == 0)
 	{
-		info->lang = GEDIT_PLUGIN_LOADER_PY;
+		info->loader = GEDIT_PLUGIN_LOADER_PY;
 #ifndef ENABLE_PYTHON
-		g_warning ("Cannot load python extension '%s', gedit was not "
-					"compiled with python support", file);
+		g_warning ("Cannot load Python plugin '%s' since gedit was not "
+			   "compiled with Python support.", file);
 		goto error;
 #endif
 	}
 	else
 	{
-		info->lang = GEDIT_PLUGIN_LOADER_C;
+		info->loader = GEDIT_PLUGIN_LOADER_C;
 	}
 	g_free (str);
 
@@ -224,7 +255,7 @@ gedit_plugins_engine_load (const gchar *file)
 	else
 		gedit_debug_message (DEBUG_PLUGINS, "Could not find 'Copyright' in %s", file);
 
-	/* Get Copyright */
+	/* Get Website */
 	str = g_key_file_get_string (plugin_file,
 				     "Gedit Plugin",
 				     "Website",
@@ -236,6 +267,10 @@ gedit_plugins_engine_load (const gchar *file)
 		
 	g_key_file_free (plugin_file);
 	
+	/* If we know nothing about the availability of the plugin,
+	   set it as available */
+	info->available = TRUE;
+	
 	return info;
 
 error:
@@ -248,6 +283,13 @@ error:
 	return NULL;
 }
 
+static gint
+compare_plugin_info (GeditPluginInfo *info1,
+		     GeditPluginInfo *info2)
+{
+	return strcmp (info1->location, info2->location);
+}
+
 static void
 gedit_plugins_engine_load_dir (const gchar *dir)
 {
@@ -281,6 +323,21 @@ gedit_plugins_engine_load_dir (const gchar *dir)
 			if (info == NULL)
 				continue;
 
+			/* If a plugin with this name has already been loaded
+			 * drop this one (user plugins override system plugins) */
+			if (g_slist_find_custom (gedit_plugins_list,
+						 info,
+						 (GCompareFunc)compare_plugin_info) != NULL)
+			{
+				g_warning ("Two or more plugins named '%s'. "
+					   "Only the first will be considered.\n",
+					   info->location);
+
+				gedit_plugin_info_free (info);
+
+				continue;
+			}
+
 			/* Actually, the plugin will be activated when reactivate_all
 			 * will be called for the first time. */
 			info->active = (g_slist_find_custom (active_plugins,
@@ -383,26 +440,7 @@ gedit_plugins_engine_shutdown (void)
 	{
 		GeditPluginInfo *info = (GeditPluginInfo*)pl->data;
 
-		if (info->plugin != NULL)
-		{
-		       	gedit_debug_message (DEBUG_PLUGINS, "Unref plugin %s", info->name);
-
-			g_object_unref (info->plugin);
-			
-			/* info->module must not be unref since it is not possible to finalize 
-			 * a type module */
-		}
-
-		g_free (info->file);
-		g_free (info->location);
-		g_free (info->name);
-		g_free (info->desc);
-		g_free (info->icon_name);
-		g_free (info->website);
-		g_free (info->copyright);
-		g_strfreev (info->authors);
-		
-		g_free (info);
+		gedit_plugin_info_free (info);
 	}
 
 	g_slist_foreach (active_plugins, (GFunc)g_free, NULL);
@@ -437,8 +475,9 @@ load_plugin_module (GeditPluginInfo *info)
 	g_return_val_if_fail (info->file != NULL, FALSE);
 	g_return_val_if_fail (info->location != NULL, FALSE);
 	g_return_val_if_fail (info->plugin == NULL, FALSE);
+	g_return_val_if_fail (info->available, FALSE);
 	
-	switch (info->lang)
+	switch (info->loader)
 	{
 		case GEDIT_PLUGIN_LOADER_C:
 			dirname = g_path_get_dirname (info->file);	
@@ -450,12 +489,32 @@ load_plugin_module (GeditPluginInfo *info)
 	
 			info->module = G_TYPE_MODULE (gedit_module_new (path));
 			g_free (path);
+			
 			break;
+
 #ifdef ENABLE_PYTHON
 		case GEDIT_PLUGIN_LOADER_PY:
 		{
-			gchar *dir = g_path_get_dirname (info->file);
+			gchar *dir;
 			
+			if (!gedit_python_init ())
+			{
+				/* Mark plugin as unavailable and fails */
+				info->available = FALSE;
+				
+				g_warning ("Cannot load Python plugin '%s' since gedit "
+				           "was not able to initialize the Python interpreter.",
+				           info->name);
+
+				return FALSE;
+			}
+
+			dir = g_path_get_dirname (info->file);
+			
+			g_return_val_if_fail ((info->location != NULL) &&
+			                      (info->location[0] != '\0'),
+			                      FALSE);
+
 			info->module = G_TYPE_MODULE (
 					gedit_python_module_new (dir, info->location));
 					
@@ -463,38 +522,55 @@ load_plugin_module (GeditPluginInfo *info)
 			break;
 		}
 #endif
+		default:
+			g_return_val_if_reached (FALSE);
 	}
 
-	
-	if (g_type_module_use (info->module) == FALSE)
+	if (!g_type_module_use (info->module))
 	{
-		switch (info->lang)
+		switch (info->loader)
 		{
 			case GEDIT_PLUGIN_LOADER_C:
-				g_warning ("Could not load plugin file at %s\n",
-				   gedit_module_get_path (GEDIT_MODULE (info->module)));
+				g_warning ("Cannot load plugin '%s' since file '%s' cannot be read.",
+					   info->name,			           
+					   gedit_module_get_path (GEDIT_MODULE (info->module)));
 				break;
+
 			case GEDIT_PLUGIN_LOADER_PY:
-				g_warning ("Could not load python module %s\n", info->location);
+				g_warning ("Cannot load Python plugin '%s' since file '%s' cannot be read.",
+					   info->name,			           
+					   info->location);
 				break;
+
+			default:
+				g_return_val_if_reached (FALSE);				
 		}
 			   
 		g_object_unref (G_OBJECT (info->module));
 		info->module = NULL;
+
+		/* Mark plugin as unavailable and fails */
+		info->available = FALSE;
 		
 		return FALSE;
 	}
 	
-	switch (info->lang)
+	switch (info->loader)
 	{
 		case GEDIT_PLUGIN_LOADER_C:
-			info->plugin = GEDIT_PLUGIN (gedit_module_new_object (GEDIT_MODULE (info->module)));
+			info->plugin = 
+				GEDIT_PLUGIN (gedit_module_new_object (GEDIT_MODULE (info->module)));
 			break;
+
 #ifdef ENABLE_PYTHON
 		case GEDIT_PLUGIN_LOADER_PY:
-			info->plugin = GEDIT_PLUGIN (gedit_python_module_new_object (GEDIT_PYTHON_MODULE (info->module)));
+			info->plugin = 
+				GEDIT_PLUGIN (gedit_python_module_new_object (GEDIT_PYTHON_MODULE (info->module)));
 			break;
 #endif
+
+		default:
+			g_return_val_if_reached (FALSE);
 	}
 	
 	g_type_module_unuse (info->module);
@@ -509,6 +585,12 @@ gedit_plugins_engine_activate_plugin_real (GeditPluginInfo *info)
 {
 	gboolean res = TRUE;
 
+	if (!info->available)
+	{
+		/* Plugin is not available, don't try to activate/load it */
+		return FALSE;
+	}
+		
 	if (info->plugin == NULL)
 		res = load_plugin_module (info);
 
@@ -524,8 +606,7 @@ gedit_plugins_engine_activate_plugin_real (GeditPluginInfo *info)
 		}
 	}
 	else
-		g_warning ("Error, impossible to activate plugin '%s'",
-			   info->name);
+		g_warning ("Error activating plugin '%s'", info->name);
 
 	return res;
 }
@@ -537,6 +618,9 @@ gedit_plugins_engine_activate_plugin (GeditPluginInfo *info)
 
 	g_return_val_if_fail (info != NULL, FALSE);
 
+	if (!info->available)
+		return FALSE;
+		
 	if (info->active)
 		return TRUE;
 
@@ -554,7 +638,7 @@ gedit_plugins_engine_activate_plugin (GeditPluginInfo *info)
 		{
 			if (strcmp (info->location, (gchar *)list->data) == 0)
 			{
-				g_warning ("Plugin %s is already active.", info->location);
+				g_warning ("Plugin '%s' is already active.", info->name);
 				return TRUE;
 			}
 
@@ -604,7 +688,7 @@ gedit_plugins_engine_deactivate_plugin (GeditPluginInfo *info)
 
 	g_return_val_if_fail (info != NULL, FALSE);
 
-	if (!info->active)
+	if (!info->active || !info->available)
 		return TRUE;
 
 	gedit_plugins_engine_deactivate_plugin_real (info);
@@ -630,7 +714,7 @@ gedit_plugins_engine_deactivate_plugin (GeditPluginInfo *info)
 
 	if (!res)
 	{
-		g_warning ("Plugin %s is already deactivated.", info->location);
+		g_warning ("Plugin '%s' is already deactivated.", info->name);
 		return TRUE;
 	}
 
@@ -651,7 +735,15 @@ gedit_plugins_engine_plugin_is_active (GeditPluginInfo *info)
 {
 	g_return_val_if_fail (info != NULL, FALSE);
 	
-	return info->active;
+	return (info->available && info->active);
+}
+
+gboolean
+gedit_plugins_engine_plugin_is_available (GeditPluginInfo *info)
+{
+	g_return_val_if_fail (info != NULL, FALSE);
+	
+	return (info->available != FALSE);
 }
 
 static void
@@ -667,13 +759,15 @@ reactivate_all (GeditWindow *window)
 		
 		GeditPluginInfo *info = (GeditPluginInfo*)pl->data;
 
-		if ((info->plugin == NULL) && info->active)
-			res = load_plugin_module (info);
-			
-		if (info->active && res)
-		{
-			gedit_plugin_activate (info->plugin,
-					       window);
+		/* If plugin is not available, don't try to activate/load it */
+		if (info->available && info->active)
+		{		
+			if (info->plugin == NULL)
+				res = load_plugin_module (info);
+				
+			if (res)
+				gedit_plugin_activate (info->plugin,
+						       window);			
 		}
 	}
 	
@@ -698,7 +792,7 @@ gedit_plugins_engine_update_plugins_ui (GeditWindow *window,
 	{
 		GeditPluginInfo *info = (GeditPluginInfo*)pl->data;
 
-		if (!info->active)
+		if (!info->available || !info->active)
 			continue;
 			
 	       	gedit_debug_message (DEBUG_PLUGINS, "Updating UI of %s", info->name);
@@ -714,7 +808,7 @@ gedit_plugins_engine_plugin_is_configurable (GeditPluginInfo *info)
 
 	g_return_val_if_fail (info != NULL, FALSE);
 
-	if ((info->plugin == NULL) || !info->active)
+	if ((info->plugin == NULL) || !info->active || !info->available)
 		return FALSE;
 	
 	return gedit_plugin_is_configurable (info->plugin);
@@ -769,7 +863,7 @@ gedit_plugins_engine_active_plugins_changed (GConfClient *client,
 	if (!((entry->value->type == GCONF_VALUE_LIST) && 
 	      (gconf_value_get_list_type (entry->value) == GCONF_VALUE_STRING)))
 	{
-		g_warning ("The gconf key '%s' mat be corrupted", GEDIT_PLUGINS_ENGINE_KEY);
+		g_warning ("The gconf key '%s' may be corrupted.", GEDIT_PLUGINS_ENGINE_KEY);
 		return;
 	}
 	
@@ -782,6 +876,9 @@ gedit_plugins_engine_active_plugins_changed (GConfClient *client,
 	{
 		GeditPluginInfo *info = (GeditPluginInfo*)pl->data;
 
+		if (!info->available)
+			continue;
+
 		to_activate = (g_slist_find_custom (active_plugins,
 						    info->location,
 						    (GCompareFunc)strcmp) != NULL);
